/*
 * A dataspace server exposing file contents from an Ext2-based filesystem.
 *
 * Copyright (C) 2020, 2021, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <ipc/thread.h>

#include <stdio.h>
#include <stdlib.h>

#include <e2access/fs.h>
#include <fsserver/page_queue_shared.h>
#include <fsserver/pages.h>
#include <fsserver/resource_server.h>
#include <fsserver/ext2_filesystem.h>
#include <mem/memory_incremental.h>
#include <systypes/env.h>



/* Default number of pages for files. */

const unsigned int MEMORY_PAGES = 20;



/* Server program. */

int main(int argc, char *argv[])
{
  /* Require filesystem object details. */

  if (argc < 3)
  {
    printf("Need a filesystem capability name and filename.\n");
    return 1;
  }

  const char *fs_cap = argv[1];
  const char *fs_filename = argv[2];
  long err;

  /* Introduce concurrency control. */

  err = ipc_thread_init();

  if (err)
  {
    printf("Initialisation error: %s\n", l4sys_errtostr(err));
    return 1;
  }

  unsigned int memory_pages = MEMORY_PAGES;

  if (argc > 3)
    memory_pages = atoi(argv[3]);

  const char *server_name = (argc > 4) ? argv[4] : ENV_FILESYSTEM_SERVER_NAME;

  /* Set the capability used to access the filesystem. */

  e2access_init(fs_cap);

  /* Attempt to open the filesystem. */

  ext2_filsys fs;
  errcode_t retval = e2access_open(fs_filename, EXT2_FLAG_RW, &fs); // EXT2_FLAG_SOFTSUPP_FEATURES | EXT2_FLAG_64BITS

  if (retval)
  {
    printf("Could not obtain filesystem: %ld.\n", retval);
    return 1;
  }

  /* Some memory plus infrastructure. */

  MemoryIncremental mem(memory_pages);
  PageQueueShared queue;
  Pages pages(&mem, &queue);
  Ext2Filesystem filesystem(&pages, fs);

  /* Register a server associating it with the given object. */

  ResourceServer server(&filesystem);
  err = server.bind(server_name);

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Starting server using %d pages...\n", memory_pages);
  server.start();
  return 0;
}
